local dir = require "pl.dir"
local path = require "pl.path"
local store = require "volksdata.store"

local term = require "volksdata.term"
local namespace = require "volksdata.namespace"


local fpath = debug.getinfo(1, "S").source:sub(2)
local root_path = path.dirname(path.dirname(fpath))
local config_path = os.getenv("PA_CONFIG_DIR") or (root_path .. "/config")


local M = {
    -- Project root path.
    root = root_path,
    config = dofile(config_path .. "/app.lua"),

    default_title = "Pocket Archive",

    --Logger config.
    logger = require "sllog":init{
      {"err",   "%T PKAR %-5L ",          "%n", io.stderr},
      {"warn",  "%T PKAR %-5L ",          "%n", io.stderr},
      {"info",  "%T PKAR %-5L ",          "%n", io.stderr},
      {"debug", "%T PKAR %-5L%f (%S) ",   "%n", io.stderr},
      timefn=(socket or {}).gettime,  -- use socket.gettime if available
      report="debug",   -- to which level should internal log events be passed?
      hookrequire=true, -- also report calls to require()
      level="debug",    -- output levels up to and including "dbg"
    },
}

M.store_id = "file://" .. (os.getenv("PA_BASE") or M.config.fs.dres_path)
M.store = store.new(store.MDB, M.store_id)

print(("Default NSM: %s"):format(M.nsm))

for pfx, ns in pairs(M.config.namespace) do namespace.add(pfx, ns) end

-- Some constant terms.
M.RDF_TYPE = term.new_iriref_ns("rdf:type", M.nsm)

-- Initialize random ID generator.
math.randomseed(M.config.id.seed[1], M.config.id.seed[2])


local par_ns = namespace.get_ns("par")

--[[
  Gnerate pairtree directory and file path from an ID string and prefix.

  The directory is created if not existing.

  The ID string can include the `par:` namespace prefix or the fully qualified
  namespace.

  ext is optional, and is appended to the raw path.

  if no_create is not nil or false, the directory will not be checked for
  existence or created.

  return: full file path, with the optional extension if provided.
--]]
M.gen_pairtree = function (pfx, id_str, ext, no_create)
    local bare_id = id_str:gsub(par_ns, ""):gsub("^par:", "")
    local res_dir = path.join(pfx, bare_id:sub(1,2), bare_id:sub(3,4))

    local created, err
    if (path.isdir(res_dir)) or no_create then created = false
    else
        created, err = dir.makepath(res_dir)
        if not created then error(err) end
    end

    local fpath
    if (res_dir) then fpath = path.join(res_dir, bare_id .. (ext or "")) end

    return fpath, created
end

return M
